import { HTMLProps, CSSProperties, ReactNode, Component } from 'react';
import { TextFieldProps } from '@mui/material/TextField';
import { AutocompleteProps } from '@mui/material/Autocomplete';
import { CheckboxProps } from '@mui/material/Checkbox';
import { DatePickerProps } from '@mui/x-date-pickers/DatePicker';
import { MuiPickersAdapter, DateTimePickerProps, TimePickerProps } from '@mui/x-date-pickers';
import { Dayjs } from 'dayjs';
import { FormControlProps } from '@mui/material/FormControl';
import { FormHelperTextProps } from '@mui/material/FormHelperText';
import { InputProps as InputProps$1 } from '@mui/material/Input';
import { InputLabelProps } from '@mui/material/InputLabel';
import { SelectProps } from '@mui/material/Select';
import { ToggleButtonGroupProps } from '@mui/material/ToggleButtonGroup';

type AutocompleteOptionType = {
    label: string
    value: string
}
type AutocompleteFragType = string | number
type AutocompleteValueType = AutocompleteOptionType[] | AutocompleteOptionType | undefined
type AutocompleteInputValueType = AutocompleteFragType | AutocompleteFragType[] | null

interface AutocompleteInputProps
    extends BaseInput<AutocompleteInputValueType>,
        Omit<AutocompleteProps<any, boolean, true, boolean, 'div'>, 'renderInput' | 'defaultValue' | 'ref'> {
    type: 'autocomplete'
    disableClearOnChangeOptions?: boolean
    variant?: Variant
    label?: string
    renderInput?: (params: TextFieldProps) => React.ReactNode
    InputProps?: TextFieldProps
    defaultValue?: AutocompleteInputValueType
    loading?: boolean
}

type CheckboxInputValueType = boolean
interface CheckboxInputProps extends BaseInput<CheckboxInputValueType>, Omit<CheckboxProps, 'defaultChecked' | 'ref'> {
    type: 'checkbox'
    label?: string
    defaultChecked?: CheckboxInputValueType
}

type CustomInputValueType = any
interface CustomInputProps extends Omit<BaseInput, 'onChangeValue' | 'ref'> {
    type: 'custom'
    element: React.ElementType
    allowObject?: boolean,
    [key: string]: any
}

type DateInputValueType = Dayjs | null
interface DateInputProps extends BaseInput<DateInputValueType>, Omit<DatePickerProps<Dayjs, false>,  'value' | 'renderInput' | 'ref'> {
    type: 'date'
    dateAdapter?: new (...args: any) => MuiPickersAdapter<Dayjs>
    variant?: Variant
    defaultValue?: DateInputValueType
    fullWidth?: boolean
    InputProps?: TextFieldProps
    // onChange?: (value: any, keyboardInputValue?: string) => void
}

type DatetimeInputValueType = Dayjs | null
interface DatetimeInputProps extends BaseInput<DatetimeInputValueType>, Omit<DateTimePickerProps<Dayjs, false>, 'value' | 'renderInput' | 'ref'> {
    type: 'datetime'
    dateAdapter?: new (...args: any) => MuiPickersAdapter<Dayjs>
    variant?: Variant
    defaultValue?: DatetimeInputValueType
    fullWidth?: boolean
    InputProps?: TextFieldProps
}

type FileInputValueType = FileList | null
interface FileInputProps extends BaseInput<FileInputValueType>, Omit<InputProps$1, 'ref'> {
    type: 'file'
    label?: string
    defaultValue?: FileInputValueType
    variant?: Variant
    helperText?: string
    FormControlProps?: FormControlProps
    InputLabelProps?: InputLabelProps
    FormHelperTextProps?: FormHelperTextProps
    multiple?: boolean
}

type GroupInputValueType = ObjectLiteral | undefined | null
interface GroupInputProps extends Omit<BaseInput<GroupInputValueType>, 'onChangeValue'> {
    type: 'group'
    inputs: InputProps[]
    itemWrapper?: (children: JSX.Element) => JSX.Element,
    onChangeValue?: (data: OutputValues) => void
}

type ItemsInputValueType = any
interface ItemsInputProps extends Omit<BaseInput<ItemsInputValueType>, 'onChangeValue'> {
    type: 'items'
    inputs: InputProps[]
    itemWrapper?: (children: JSX.Element) => JSX.Element
    renderHeader?: (addItem: Function, removeAll: Function) => JSX.Element
    removeIcon?: JSX.Element | React.ReactNode | boolean
    copyIcon?: JSX.Element | React.ReactNode | boolean
    minItems?: number
    maxItems?: number
}

type MaskInputValueType = string | null
interface MaskInputProps extends BaseInput<MaskInputValueType>, Omit<TextFieldProps, 'defaultValue' | 'ref'> {
    type: 'mask'
    pattern: string
    /**
     * @default _
     */
    char?: string
    defaultValue?: MaskInputValueType
}

type MobileInputValueType = string | null
interface MobileInputProps extends BaseInput<MobileInputValueType>, Omit<TextFieldProps, 'defaultValue' | 'ref'> {
    type: 'mobile'
    defaultValue?: MobileInputValueType
}

type NumberInputValueType = number | null
interface NumberInputProps extends BaseInput<NumberInputValueType>, Omit<TextFieldProps, 'defaultValue' | 'ref'> {
    type: 'number'
    defaultValue?: NumberInputValueType
}

type OtpInputValueType = string | null
interface OtpInputProps extends BaseInput<OtpInputValueType>, Omit<HTMLProps<HTMLInputElement>, 'defaultValue' | 'ref'> {
    type: 'otp'

    defaultValue?: OtpInputValueType

    /** * @default standard */
    variant?: 'outlined' | 'standard'

    /** * @default 4 */
    digits?: number

    autoFocus?: boolean

    /** @default 55 */
    height?: number

    /** @default 34.61 */
    width?: number

    /** @default 1 */
    borderWidth?: number

    /** @default 8 */
    spaceBetween?: number

    /** @default 4 */
    borderRadius?: number

    /** @default #0000003b */
    emptyColor?: CSSProperties['color']

    /** @default #1976d2 */
    focusColor?: CSSProperties['color']

    /** @default #68b36b */
    completeColor?: CSSProperties['color']

    /** @default 400 */
    fontWeight?: number

    /** @default 23 */
    fontSize?: number

    /** @default #323232 */
    color?: CSSProperties['color']
}

type PasswordInputValueType = string | null
interface PasswordInputProps extends BaseInput<PasswordInputValueType>, Omit<TextFieldProps, 'defaultValue' | 'ref'> {
    type: 'password'
    defaultValue?: PasswordInputValueType
}

type SelectInputValueType = any
type SelectOptionType = {
    label: string | JSX.Element
    value: any
}
interface SelectInputProps extends BaseInput<SelectInputValueType>, Omit<SelectProps, 'defaultValue' | 'ref'> {
    type: 'select'
    options: SelectOptionType[]
    defaultValue?: SelectInputValueType
    listSubheaderText?: string
    loading?: boolean
}

type TextInputValueType = string | null
interface TextInputProps extends BaseInput<TextInputValueType>, Omit<TextFieldProps, 'defaultValue' | 'ref'> {
    type: 'text'
    defaultValue?: TextInputValueType
}

type TimeInputValueType = Dayjs | null
interface TimeInputProps extends BaseInput<TimeInputValueType>, Omit<TimePickerProps<Dayjs, false>, 'value' | 'renderInput' | 'ref'> {
    type: 'time'
    dateAdapter?: new (...args: any) => MuiPickersAdapter<Dayjs>
    variant?: Variant
    defaultValue?: TimeInputValueType
    fullWidth?: boolean
    InputProps?: TextFieldProps
}

type ToggleInputValueType = string | string[] | null
type ToggleInputOptionType = {
    label: string | ReactNode
    value: any
}

interface ToggleInputProps extends BaseInput<ToggleInputValueType>, Omit<ToggleButtonGroupProps, 'defaultValue' | 'ref'> {
    type: 'toggle'
    options: ToggleInputOptionType[]
    defaultValue?: ToggleInputValueType
    exclusive?: boolean
    label?: string
    enforceValueSet?: boolean
}

type InputProps =
    | TextInputProps
    | NumberInputProps
    | ItemsInputProps
    | CustomInputProps
    | CheckboxInputProps
    | MobileInputProps
    | OtpInputProps
    | PasswordInputProps
    | SelectInputProps
    | DateInputProps
    | TimeInputProps
    | DatetimeInputProps
    | MaskInputProps
    | FileInputProps
    | AutocompleteInputProps
    | ToggleInputProps
    | GroupInputProps

type ObjectLiteral = { [key: string]: any }
type Variant = 'standard' | 'filled' | 'outlined'

interface InputActions<ValueType = any> {
    setValue: (data: ValueType) => Promise<ValueType>
    getValue: () => Promise<ValueType>
    clear: () => Promise<ValueType>
    click: () => Promise<any>
    focus: () => Promise<any>
    blur: () => Promise<any>
    copy?: () => Promise<any>
}

interface BaseInput<ValueType = any> {
    selector: string
    required?: boolean
    /**
     * @default true
     */
    visible?: ((e: any) => boolean | undefined) | boolean
    wrapper?: (children: JSX.Element, actions: InputActions<ValueType>) => JSX.Element
    onChangeValue?: (value: ValueType) => void
    getMutator?: (value: any) => any
    setMutator?: (value: any) => any
    formatter?: (value: any) => any
    _call_parent_for_update?: () => Promise<boolean>
    ref?: any
}

type OutputValues<T = ObjectLiteral> = {
    data: T
    validation: {
        status: boolean
        inputs: Array<any>
    }
}

interface FormBuilderImplements {
    getValues: (validation: boolean) => OutputValues;
    setValues: (values: ObjectLiteral) => Promise<void>;
    clear: () => Promise<void>;
}
interface IState {
    isMounted: boolean;
    time: number | null;
    inInternalSettingProcess: boolean;
}
interface FormBuilderProps {
    inputs: InputProps[];
    onChange?: Function;
    onMount?: Function;
}
declare class FormBuilder extends Component<FormBuilderProps, IState> implements FormBuilderImplements {
    state: IState;
    private inputRefs;
    private inputs;
    private defaultValues;
    private didMountEvent;
    componentDidMount(): void;
    componentWillUnmount(): void;
    componentDidUpdate(): void;
    getValues: (validation?: boolean) => OutputValues;
    private setObjectValues;
    private setNormalValue;
    private setValue;
    setValues: (value: ObjectLiteral) => Promise<any>;
    private syncSetValues;
    clear: () => Promise<any>;
    private lastVisibilityOfInputs;
    private checkVisibility;
    private onUpdateInputs;
    private executeAction;
    private renderInput;
    render(): JSX.Element;
}

export { type AutocompleteInputProps, type AutocompleteOptionType, type AutocompleteValueType, type CheckboxInputProps, type CheckboxInputValueType, type CustomInputProps, type CustomInputValueType, type DateInputProps, type DateInputValueType, type DatetimeInputProps, type DatetimeInputValueType, type FileInputProps, type FileInputValueType, FormBuilder, type GroupInputProps, type GroupInputValueType, type ItemsInputProps, type ItemsInputValueType, type MaskInputProps, type MaskInputValueType, type MobileInputProps, type MobileInputValueType, type NumberInputProps, type NumberInputValueType, type OtpInputProps, type OtpInputValueType, type PasswordInputProps, type PasswordInputValueType, type SelectInputProps, type SelectInputValueType, type TextInputProps, type TextInputValueType, type TimeInputProps, type TimeInputValueType, type ToggleInputProps, type ToggleInputValueType, FormBuilder as default };
